@@ -1,14 +1,9 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import logging
 import re
 from string import Template
 
-import six
-
 from .errors import ConfigurationError
-from compose.const import COMPOSEFILE_V2_0 as V2_0
+from compose.const import COMPOSEFILE_V1 as V1
 from compose.utils import parse_bytes
 from compose.utils import parse_nanoseconds_int
 
@@ -16,7 +11,7 @@ from compose.utils import parse_nanoseconds_int
 log = logging.getLogger(__name__)
 
 
-class Interpolator(object):
+class Interpolator:
 
     def __init__(self, templater, mapping):
         self.templater = templater
@@ -30,21 +25,21 @@ class Interpolator(object):
 
 
 def interpolate_environment_variables(version, config, section, environment):
-    if version <= V2_0:
+    if version == V1:
         interpolator = Interpolator(Template, environment)
     else:
         interpolator = Interpolator(TemplateWithDefaults, environment)
 
     def process_item(name, config_dict):
-        return dict(
-            (key, interpolate_value(name, key, val, section, interpolator))
+        return {
+            key: interpolate_value(name, key, val, section, interpolator)
             for key, val in (config_dict or {}).items()
-        )
+        }
 
-    return dict(
-        (name, process_item(name, config_dict or {}))
+    return {
+        name: process_item(name, config_dict or {})
         for name, config_dict in config.items()
-    )
+    }
 
 
 def get_config_path(config_key, section, name):
@@ -77,13 +72,13 @@ def recursive_interpolate(obj, interpolator, config_path):
     def append(config_path, key):
         return '{}/{}'.format(config_path, key)
 
-    if isinstance(obj, six.string_types):
+    if isinstance(obj, str):
         return converter.convert(config_path, interpolator.interpolate(obj))
     if isinstance(obj, dict):
-        return dict(
-            (key, recursive_interpolate(val, interpolator, append(config_path, key)))
-            for (key, val) in obj.items()
-        )
+        return {
+            key: recursive_interpolate(val, interpolator, append(config_path, key))
+            for key, val in obj.items()
+        }
     if isinstance(obj, list):
         return [recursive_interpolate(val, interpolator, config_path) for val in obj]
     return converter.convert(config_path, obj)
@@ -91,17 +86,17 @@ def recursive_interpolate(obj, interpolator, config_path):
 
 class TemplateWithDefaults(Template):
     pattern = r"""
-        %(delim)s(?:
-            (?P<escaped>%(delim)s) |
-            (?P<named>%(id)s)      |
-            {(?P<braced>%(bid)s)}  |
+        {delim}(?:
+            (?P<escaped>{delim}) |
+            (?P<named>{id})      |
+            {{(?P<braced>{bid})}}  |
             (?P<invalid>)
         )
-        """ % {
-        'delim': re.escape('$'),
-        'id': r'[_a-z][_a-z0-9]*',
-        'bid': r'[_a-z][_a-z0-9]*(?:(?P<sep>:?[-?])[^}]*)?',
-    }
+        """.format(
+        delim=re.escape('$'),
+        id=r'[_a-z][_a-z0-9]*',
+        bid=r'[_a-z][_a-z0-9]*(?:(?P<sep>:?[-?])[^}]*)?',
+    )
 
     @staticmethod
     def process_braced_group(braced, sep, mapping):
@@ -116,12 +111,14 @@ class TemplateWithDefaults(Template):
             var, _, err = braced.partition(':?')
             result = mapping.get(var)
             if not result:
+                err = err or var
                 raise UnsetRequiredSubstitution(err)
             return result
         elif '?' == sep:
             var, _, err = braced.partition('?')
             if var in mapping:
                 return mapping.get(var)
+            err = err or var
             raise UnsetRequiredSubstitution(err)
 
     # Modified from python2.7/string.py
@@ -138,9 +135,9 @@ class TemplateWithDefaults(Template):
 
             if named is not None:
                 val = mapping[named]
-                if isinstance(val, six.binary_type):
+                if isinstance(val, bytes):
                     val = val.decode('utf-8')
-                return '%s' % (val,)
+                return '{}'.format(val)
             if mo.group('escaped') is not None:
                 return self.delimiter
             if mo.group('invalid') is not None:
@@ -177,7 +174,7 @@ def service_path(*args):
 
 
 def to_boolean(s):
-    if not isinstance(s, six.string_types):
+    if not isinstance(s, str):
         return s
     s = s.lower()
     if s in ['y', 'yes', 'true', 'on']:
@@ -188,11 +185,11 @@ def to_boolean(s):
 
 
 def to_int(s):
-    if not isinstance(s, six.string_types):
+    if not isinstance(s, str):
         return s
 
     # We must be able to handle octal representation for `mode` values notably
-    if six.PY3 and re.match('^0[0-9]+$', s.strip()):
+    if re.match('^0[0-9]+$', s.strip()):
         s = '0o' + s[1:]
     try:
         return int(s, base=0)
@@ -201,7 +198,7 @@ def to_int(s):
 
 
 def to_float(s):
-    if not isinstance(s, six.string_types):
+    if not isinstance(s, str):
         return s
 
     try:
@@ -224,12 +221,12 @@ def bytes_to_int(s):
 
 
 def to_microseconds(v):
-    if not isinstance(v, six.string_types):
+    if not isinstance(v, str):
         return v
     return int(parse_nanoseconds_int(v) / 1000)
 
 
-class ConversionMap(object):
+class ConversionMap:
     map = {
         service_path('blkio_config', 'weight'): to_int,
         service_path('blkio_config', 'weight_device', 'weight'): to_int,
@@ -246,6 +243,8 @@ class ConversionMap(object):
         service_path('healthcheck', 'disable'): to_boolean,
         service_path('deploy', 'labels', PATH_JOKER): to_str,
         service_path('deploy', 'replicas'): to_int,
+        service_path('deploy', 'placement', 'max_replicas_per_node'): to_int,
+        service_path('deploy', 'resources', 'limits', "cpus"): to_float,
         service_path('deploy', 'update_config', 'parallelism'): to_int,
         service_path('deploy', 'update_config', 'max_failure_ratio'): to_float,
         service_path('deploy', 'rollback_config', 'parallelism'): to_int,
