@@ -1,7 +1,3 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
-
 class OperationFailedError(Exception):
     def __init__(self, reason):
         self.msg = reason
@@ -19,15 +15,20 @@ class HealthCheckException(Exception):
 
 class HealthCheckFailed(HealthCheckException):
     def __init__(self, container_id):
-        super(HealthCheckFailed, self).__init__(
+        super().__init__(
             'Container "{}" is unhealthy.'.format(container_id)
         )
 
 
 class NoHealthCheckConfigured(HealthCheckException):
     def __init__(self, service_name):
-        super(NoHealthCheckConfigured, self).__init__(
+        super().__init__(
             'Service "{}" is missing a healthcheck configuration'.format(
                 service_name
             )
         )
+
+
+class CompletedUnsuccessfully(Exception):
+    def __init__(self, container_id, exit_code):
+        self.msg = 'Container "{}" exited with code {}.'.format(container_id, exit_code)
